package
{

	import away3d.containers.View3D;
	import away3d.core.base.Mesh;
	import away3d.core.math.Number3D;
	import away3d.debug.AwayStats;
	import away3d.materials.BitmapMaterial;
	import away3d.materials.GlassMaterial;
	import away3d.primitives.Skybox6;
	import away3d.primitives.Sphere;
	import away3d.primitives.Torus;
	import away3d.primitives.TorusKnot;

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;

	[SWF(width="800", height="600", frameRate="40", backgroundColor="0x000000")]
	public class GlassMaterialDemo extends Sprite
	{
		private const GLASS_COLOURS : Array = [ 0xffffff, 0x10aa20, 0xff1020, 0xff1020 ];
		private const ENV_MAP_ALPHAS : Array = [ .25, .4, .6, .75 ];
		private const CHROMATIC_ABERRATION : Array = [ true, true, false, false ];

		private const CAMERA_DISTANCE : Number = 1000;

		// source textures
		[Embed(source="../assets/cube6hills.jpg")]
		private const SKY_BOX : Class;

		[Embed(source="../assets/sphereMap_hills.jpg")]
		private const SPHERE_MAP : Class;

		[Embed(source="../assets/sphereNormals.png")]
		private const SPHERE_NORMALS : Class;

		[Embed(source="../assets/torusNormals.png")]
		private const TORUS_NORMALS : Class;

		[Embed(source="../assets/knotNormals.png")]
		private const KNOT_NORMALS : Class;

		private const CAM_TARGET : Number3D = new Number3D(0, 0, 0);

		private var _view : View3D;
		private var _sign : Bitmap;

		private var _count : int;

		private var _models : Array;
		private var _materials : Array;

		private var _envMap : BitmapData;

		public function GlassMaterialDemo()
		{
			initData();
			initView();
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(MouseEvent.CLICK, onClick);
		}

		private function initData():void
		{
			_models = [];
			_materials = [];
			_envMap = new SPHERE_MAP().bitmapData;
		}

		/**
		 * Cycle through views
		 */
		private function onClick(event:MouseEvent):void
		{
			_view.scene.removeChild(_models[_count]);
			if (++_count == 4) _count = 0;

			setDemoCycle(_count);
		}

		/**
		 * Creates all the items used in the view
		 */
		private function initView() : void
		{
			var tf : TextField = new TextField();
			tf.textColor = 0xffffff;
			tf.text = "CLICK - Cycle model/material"
			tf.width = tf.textWidth + 10;
			tf.x = stage.stageWidth - tf.width - 10;
			tf.y = 10;

			_view = new View3D();
			_view.x = stage.stageWidth * .5;
			_view.y = stage.stageHeight * .5;
			addChild(_view);
			addChild(tf);
			addChild(new AwayStats(_view));

			initSig();
			initSkybox();
			setDemoCycle(0);
		}

		/**
		 * Adds Away3D signature
		 */
		private function initSig():void
		{
			var spr : signature = new signature();
            var bmd : BitmapData = new BitmapData(spr.width, spr.height, true, 0x00000000);
            bmd.draw(spr);
            _sign = new Bitmap(bmd);
            addChild(_sign);
            _sign.y = stage.stageHeight-_sign.height;
		}

		/**
		 * Creates the skybox
		 */
		private function initSkybox() : void
		{
			var tex : BitmapData = new SKY_BOX().bitmapData;
			var mat : BitmapMaterial = new BitmapMaterial(tex, {smooth: true});
			var skybox : Skybox6 = new Skybox6(mat);
			skybox.scale(0.0025);
			_view.scene.addChild(skybox);
		}

		/**
		 * Select a new model/material combination
		 */
		private function setDemoCycle(index:int):void
		{
			var model : Mesh;

			// create model and material if it doesn't exist yet
			if (!_models[_count])
				_models[_count] = createModel(_count);

			if (!_materials[_count])
				_materials[_count] = createMaterial(_count);

			model = _models[_count];
			model.material = _materials[_count];
			
			_view.scene.addChild(model);
		}

		/**
		 * Creates the main model
		 */
		private function createModel(index : int) : Mesh
		{
			switch (index) {
				case 0:
					return new Sphere({radius: 200, segmentsH: 10, segmentsW: 15});
					break;
				case 1:
					return _models[0];
					break;
				case 2:
					return new Torus({radius: 150, tube: 75, segmentsR: 15, segmentsT: 13});
					break;
				case 3:
					return new TorusKnot({radius: 150, tube: 45, segmentsR: 60, segmentsT: 6, p: 2, q: 3})
					break;
				default:
					return null;
			}
		}

		/**
		 * Creates the Fresnel material (this is the relevant part)
		 */
		private function createMaterial(index : int) : GlassMaterial
		{
			var normalMap : BitmapData;         

			switch(index) {
				case 0:
				case 1:
					normalMap = new SPHERE_NORMALS().bitmapData;
					break;
				case 2:                                   
					normalMap = new TORUS_NORMALS().bitmapData;
					break;
				case 3:
					normalMap = new KNOT_NORMALS().bitmapData;
					break;
				break;
			}

			return new GlassMaterial(	normalMap, _envMap, _models[index], CHROMATIC_ABERRATION[index],
										{ 	envMapAlpha: ENV_MAP_ALPHAS[index],
											innerRefraction: 2,
											glassColor: GLASS_COLOURS[index],
											smooth: true
										}
									);
		}

		/**
		 * Updates the camera position and renders               
		 */
		private function onEnterFrame(event : Event) : void
		{
			// just a random camera update
			var angX : Number = -_view.mouseX / 250;
			var angY : Number = _view.mouseY / 150;
			var x : Number, y : Number, z : Number;
			var len : Number;

			x = Math.sin(angX)*CAMERA_DISTANCE;
			y = Math.sin(angY)*CAMERA_DISTANCE;
			z = -(Math.cos(angX)+Math.cos(angY))*.125*CAMERA_DISTANCE;

			len = CAMERA_DISTANCE/Math.sqrt(x*x + y*y + z*z);

			_view.camera.x = x*len;
			_view.camera.y = y*len;
			_view.camera.z = z*len;
			_view.camera.lookAt(CAM_TARGET);

			_view.render();
		}
	}
}